@@ -1,10 +1,13 @@
+# (c) Sound Object Logic 2000-2001
+
 use strict;
 
 use Tangram::AbstractSet;
 
 package Tangram::IntrSet;
 
-use base qw( Tangram::AbstractSet );
+use vars qw(@ISA);
+ @ISA = qw( Tangram::AbstractSet );
 
 use Carp;
 
@@ -22,7 +25,7 @@ sub reschema
 			$members->{$member} = $def;
 		}
 
-		$def->{coll} ||= $class . "_$member";
+		$def->{coll} ||= $schema->{normalize}->($class) . "_$member";
 
 		$schema->{classes}{$def->{class}}{stateless} = 0;
 
@@ -30,9 +33,12 @@ sub reschema
 		{
 			my $back = $def->{back} ||= $def->{coll};
 			$schema->{classes}{ $def->{class} }{members}{backref}{$back} =
-			{
-			 col => $def->{coll} };
-			die '***' unless $schema->{types}{backref};
+			  bless {
+					 name => $back,
+					 col => $def->{coll},
+					 class => $class,
+					 field => $member
+					}, 'Tangram::BackRef';
 		}
 	}
    
@@ -40,55 +46,52 @@ sub reschema
 }
 
 sub defered_save
-{
-	my ($self, $storage, $obj, $members, $coll_id) = @_;
-
+  {
+	my ($self, $storage, $obj, $field, $def) = @_;
+	
+	return  if tied $obj->{$field};
+	
+	my $coll_id = $storage->export_object($obj);
 	my $classes = $storage->{schema}{classes};
-
-	foreach my $member (keys %$members)
-	{
-		next if tied $obj->{$member};
-		next unless exists $obj->{$member} && defined $obj->{$member};
-
-		my $def = $members->{$member};
-      
-		my $item_classdef = $classes->{$def->{class}};
-		my $table = $item_classdef->{table};
-		my $item_col = $def->{coll};
-      
-		$self->update($storage, $obj, $member,
-					  sub
-					  {
-						  my $sql = "UPDATE $table SET $item_col = $coll_id WHERE id = @_";
-						  $storage->sql_do($sql);
-					  },
-
-					  sub
-					  {
-						  my $sql = "UPDATE $table SET $item_col = NULL WHERE id = @_ AND $item_col = $coll_id";
-						  $storage->sql_do($sql);
-					  } );
-	}
-}
+	my $item_classdef = $classes->{$def->{class}};
+	my $table = $item_classdef->{table};
+	my $item_col = $def->{coll};
+	
+	$self->update($storage, $obj, $field,
+				  sub
+				  {
+					my $sql = "UPDATE $table SET $item_col = $coll_id WHERE $storage->{schema}{sql}{id_col} = @_";
+					$storage->sql_do($sql);
+				  },
+				  
+				  sub
+				  {
+					my $sql = "UPDATE $table SET $item_col = NULL WHERE $storage->{schema}{sql}{id_col} = @_ AND $item_col = $coll_id";
+					$storage->sql_do($sql);
+				  } );
+  }
 
 sub demand
-{
+  {
 	my ($self, $def, $storage, $obj, $member, $class) = @_;
 
 	my $set = Set::Object->new();
 
-	if (my $prefetch = $storage->{PREFETCH}{$class}{$member}{$storage->id($obj)})
+	if (my $prefetch = $storage->{PREFETCH}{$class}{$member}{$storage->export_object($obj)})
 	{
+	    print $Tangram::TRACE "demanding ".$storage->id($obj)
+		.".$member from prefetch\n" if $Tangram::TRACE;
 		$set->insert(@$prefetch);
 	}
 	else
 	{
-		print $Tangram::TRACE "loading $member\n" if $Tangram::TRACE;
+	    print $Tangram::TRACE "demanding ".$storage->id($obj)
+		.".$member from storage\n" if $Tangram::TRACE;
 
 		my $cursor = Tangram::CollCursor->new($storage, $def->{class}, $storage->{db});
 
-		my $coll_id = $storage->id($obj);
-		my $tid = $cursor->{-stored}->leaf_table;
+		my $coll_id = $storage->export_object($obj);
+		my $tid = $cursor->{TARGET}->object->{table_hash}{$def->{class}}; # leaf_table;
 		$cursor->{-coll_where} = "t$tid.$def->{coll} = $coll_id";
    
 		$set->insert($cursor->select);
@@ -103,6 +106,8 @@ sub erase
 {
 	my ($self, $storage, $obj, $members, $coll_id) = @_;
 
+	$coll_id = $storage->{export_id}->($coll_id);
+
 	foreach my $member (keys %$members)
 	{
 		my $def = $members->{$member};
@@ -113,7 +118,7 @@ sub erase
 		}
 		else
 		{
-			my $item_classdef = $storage->{schema}{$def->{class}};
+			my $item_classdef = $storage->{schema}{classes}{$def->{class}};
 			my $table = $item_classdef->{table} || $def->{class};
 			my $item_col = $def->{coll};
 			$storage->sql_do("UPDATE $table SET $item_col = NULL WHERE $item_col = $coll_id");
@@ -127,21 +132,20 @@ sub query_expr
 	map { Tangram::IntrCollExpr->new($obj, $_); } values %$members;
 }
 
+sub remote_expr
+{
+	my ($self, $obj, $tid) = @_;
+	Tangram::IntrCollExpr->new($obj, $self);
+}
+
 sub prefetch
 {
 	my ($self, $storage, $def, $coll, $class, $member, $filter) = @_;
-   
+
 	my $ritem = $storage->remote($def->{class});
 
 	my $prefetch = $storage->{PREFETCH}{$class}{$member} ||= {}; # weakref
 
-	my $ids = $storage->my_select_data( cols => [ $coll->{id} ], filter => $filter );
-
-	while (my ($id) = $ids->fetchrow)
-	{
-		$prefetch->{$id} = []
-	}
-
 	my $includes = $coll->{$member}->includes($ritem);
 	$includes &= $filter if $filter;
 
@@ -150,13 +154,18 @@ sub prefetch
 	while (my $item = $cursor->current)
 	{
 		my ($coll_id) = $cursor->residue;
-		push @{ $prefetch->{$coll_id} }, $item;
+		push @{ $prefetch->{$coll_id}||=[] }, $item;
 		$cursor->next;
 	}
 
 	return $prefetch;
 }
 
+sub get_intrusions {
+  my ($self, $context) = @_;
+  return [ $self->{class}, $context->{mapping}->get_home_table($self->{class}) ];
+}
+
 $Tangram::Schema::TYPES{iset} = Tangram::IntrSet->new;
 
 1;
