@@ -1,10 +1,13 @@
+# (c) Sound Object Logic 2000-2001
+
 use strict;
 
 use Tangram::AbstractSet;
 
 package Tangram::Set;
 
-use base qw( Tangram::AbstractSet );
+use vars qw(@ISA);
+ @ISA = qw( Tangram::AbstractSet );
 
 use Carp;
 
@@ -22,7 +25,7 @@ sub reschema
 			$members->{$member} = $def;
 		}
 
-		$def->{table} ||= $def->{class} . "_$member";
+		$def->{table} ||= $schema->{normalize}->($def->{class} . "_$member", 'tablename');
 		$def->{coll} ||= 'coll';
 		$def->{item} ||= 'item';
 	}
@@ -32,33 +35,26 @@ sub reschema
 
 sub defered_save
 {
-	my ($self, $storage, $obj, $members, $coll_id) = @_;
+	my ($self, $storage, $obj, $field, $def) = @_;
 
-	foreach my $member (keys %$members)
-	{
-		next if tied $obj->{$member};
-		next unless exists $obj->{$member} && defined $obj->{$member};
-
-		my $def = $members->{$member};
+	return if tied $obj->{$field};
       
-		my $table = $def->{table} || $def->{class} . "_$member";
-		my $coll_col = $def->{coll} || 'coll';
-		my $item_col = $def->{item} || 'item';
+	my $coll_id = $storage->export_object($obj);
+	my $table = $def->{table};
+	my $coll_col = $def->{coll};
+	my $item_col = $def->{item};
       
-		$self->update($storage, $obj, $member,
-					  sub
-					  {
-						  my $sql = "INSERT INTO $table ($coll_col, $item_col) VALUES ($coll_id, @_)";
-						  $storage->sql_do($sql);
-					  },
-
-					  sub
-					  {
-						  my $sql = "DELETE FROM $table WHERE $coll_col = $coll_id AND $item_col = @_";
-						  $storage->sql_do($sql);
-					  } );
-	}
-}
+	$self->update($storage, $obj, $field,
+				  sub {
+					my $sql = "INSERT INTO $table ($coll_col, $item_col) VALUES ($coll_id, @_)";
+					$storage->sql_do($sql);
+				  },
+				  sub
+				  {
+					my $sql = "DELETE FROM $table WHERE $coll_col = $coll_id AND $item_col = @_";
+					$storage->sql_do($sql);
+				  } );
+  }
 
 sub prefetch
 {
@@ -94,27 +90,29 @@ sub demand
 {
 	my ($self, $def, $storage, $obj, $member, $class) = @_;
 
-	print $Tangram::TRACE "loading $member\n" if $Tangram::TRACE;
-
 	my $set = Set::Object->new;
 
-	if (my $prefetch = $storage->{PREFETCH}{$class}{$member}{$storage->id($obj)})
+	if (my $prefetch = $storage->{PREFETCH}{$class}{$member}{$storage->export_object($obj)})
 	{
+	    print $Tangram::TRACE "getting ".$storage->id($obj)
+		.".$member from prefetch\n" if $Tangram::TRACE;
 		$set->insert(@$prefetch);
 	}
 	else
 	{
+	    print $Tangram::TRACE "demanding ".$storage->id($obj)
+		.".$member from storage\n" if $Tangram::TRACE;
 		my $cursor = Tangram::CollCursor->new($storage, $def->{class}, $storage->{db});
 
-		my $coll_id = $storage->id($obj);
+		my $coll_id = $storage->export_object($obj);
 		my $coll_tid = $storage->alloc_table;
 		my $table = $def->{table};
-		my $item_tid = $cursor->{-stored}->root_table;
+		my $item_tid = $cursor->{TARGET}->object->root_table;
 		my $coll_col = $def->{coll} || 'coll';
 		my $item_col = $def->{item} || 'item';
 		$cursor->{-coll_tid} = $coll_tid;
-		$cursor->{-coll_from} = ", $table t$coll_tid";
-		$cursor->{-coll_where} = "t$coll_tid.$coll_col = $coll_id AND t$coll_tid.$item_col = t$item_tid.id";
+		$cursor->{-coll_from} = "$table t$coll_tid";
+		$cursor->{-coll_where} = "t$coll_tid.$coll_col = $coll_id AND t$coll_tid.$item_col = t$item_tid.$storage->{schema}{sql}{id_col}";
 
 		$set->insert($cursor->select);
 	}
@@ -128,6 +126,8 @@ sub erase
 {
 	my ($self, $storage, $obj, $members, $coll_id) = @_;
 
+	$coll_id = $storage->{export_id}->($coll_id);
+
 	foreach my $member (keys %$members)
 	{
 		my $def = $members->{$member};
@@ -156,6 +156,12 @@ sub query_expr
 	map { Tangram::CollExpr->new($obj, $_); } values %$members;
 }
 
+sub remote_expr
+{
+	my ($self, $obj, $tid) = @_;
+	Tangram::CollExpr->new($obj, $self);
+}
+
 $Tangram::Schema::TYPES{set} = Tangram::Set->new;
 
 1;
